#include<math.h>
#include "mex.h"

/*Theta candiadate solutions: solutions which are closest to a reference directions within the given theta threshold.
The non-dominated solutions among these solutions are tagged to its specific reference direction based on
closest distance form solution to reference direction i.e., d2
*/
void associateThetaCandidateSolutions_MEX(int m, int Np, int * d_index, double * d_fobj, double*  directions, double * d_ideal, double * d_nadir, double th_t)
{
    int tid, i, j;
	for (tid = 0; tid < Np; tid++)
	{
		// each thread pick an index
		int index = -1;
		//each thread pick a reference direction
		double * w = &directions[tid*m];

		for (i = 0; i < Np; i++)
		{
			double dotVal = 0, nrmValObj = 0, nrmValW = 0;
			double * f = &d_fobj[i*m];

			for (j = 0; j < m; j++)
			{
				//dot product between weight and objective
				double ObjVal = (f[j] - d_ideal[j]) / (d_nadir[j] - d_ideal[j]);
				dotVal = dotVal + ObjVal* w[j];

				//norm of objective values
				nrmValObj = nrmValObj + ObjVal*ObjVal;
				//norm weight directions
				nrmValW = nrmValW + w[j] * w[j];
			}

			nrmValObj = sqrt(nrmValObj);
			nrmValW = sqrt(nrmValW);

			if (acos((dotVal / (nrmValObj*nrmValW))) <= th_t)
			{

				if (index == -1)
				{
					index = i;
				}
				else
				{
					//dominance count
					int count = 0;
					double * f_pre = &d_fobj[index*m];
					for (j = 0; j < m; j++){
						if (f[j] < f_pre[j])
							count++;
					}
					//condition based on dominance if new solution dominantes previous solution; store the index
					if (count == m)
					{
						index = i;
					}
					//if the solution is non-dominated then; store the index of one with minimum d2
					else if (count >= 1){

						double d1_n = 0, d2_n = 0, d1_p = 0, d2_p = 0, sum_n = 0, sum_p = 0;
						for (j = 0; j < m; j++)
						{
							double f_n = ((f[j] - d_ideal[j]) / (d_nadir[j] - d_ideal[j]));
							sum_n = sum_n + f_n*f_n;
							double f_p = ((f_pre[j] - d_ideal[j]) / (d_nadir[j] - d_ideal[j]));
							sum_p = sum_p + f_p*f_p;


							d1_n = d1_n + ((w[j] / nrmValW) * f_n);
							d1_p = d1_p + ((w[j] / nrmValW) * f_p);
						}
						d2_p = sqrt((sum_p)-(d1_p*d1_p));
						d2_n = sqrt((sum_n)-(d1_n*d1_n));

						if (d2_n < d2_p)
							index = i;
					}
				}
			}
		}
		//each thread reports the index found
		if(index!=-1)
            d_index[tid] = index+1;
		else
		    d_index[tid] = index;
	}
}

/* Matlab Mex interface*/

void mexFunction(int nlhs, mxArray *plhs[], int nrhs,
                 const mxArray *prhs[])
{

    if(nrhs != 5)
	{
		mexErrMsgTxt("Five inputs required.");
	} else if(nlhs != 1)
	{
		mexErrMsgTxt("One output argument");
	}
    /*input data*/
    double * f;
    double * w;
    double * ideal;
    double * nadir;
    double threshold;
    /*output data*/
    int * outIndex;

    // matlab reads the data in column major order
    // N represent the number of rows in original data
    int N = mxGetN(prhs[0]);
    int m = mxGetM(prhs[0]);
    // read the data pointer to the specific arrays: Column major order for a matirx
    f=mxGetPr(prhs[0]);
    w=mxGetPr(prhs[1]);
    ideal=mxGetPr(prhs[2]);
    nadir=mxGetPr(prhs[3]);
    threshold=mxGetScalar(prhs[4]);
    // set the output matrix
    plhs[0]=mxCreateNumericMatrix(N, 1, mxINT32_CLASS, mxREAL);
    // cast the output matrix
    outIndex=(int*) mxGetData(plhs[0]);
    // call the function
    associateThetaCandidateSolutions_MEX(m, N, outIndex, f, w, ideal, nadir, threshold);

}




